"""10 10 winter phone plugin"""

# pylint: disable=invalid-name, unused-argument, unused-variable, c-extension-no-member, unused-import

import datetime

from pathlib import Path

from lib.plugin_common import gen_adif, get_points, online_score_xml
from lib.version import __version__

name = "10 10 WINTER PHONE"
cabrillo_name = "10-10"
mode = "SSB"  # CW SSB BOTH RTTY

# 1 once per contest, 2 work each band, 3 each band/mode, 4 no dupe checking
dupe_type = 1


def show_mults(self):
    """Return display string for mults"""
    return 0


def show_qso(self):
    """Return qso count"""
    result = self.database.fetch_qso_count()
    if result:
        return int(result.get("qsos", 0))
    return 0


def calc_score(self):
    """Return calculated score"""
    result = self.database.fetch_points()
    if result is not None:
        score = result.get("Points", "0")
        if score is None:
            score = "0"
        contest_points = int(score)
        return contest_points
    return 0


def adif(self):
    """Call the generate ADIF function"""
    gen_adif(self, cabrillo_name)


def output_cabrillo_line(line_to_output, ending, file_descriptor, file_encoding):
    """"""
    print(
        line_to_output.encode(file_encoding, errors="ignore").decode(),
        end=ending,
        file=file_descriptor,
    )


def cabrillo(self, file_encoding):
    """Generates Cabrillo file. Maybe."""
    # https://www.cqwpx.com/cabrillo.htm
    # logger.debug("******Cabrillo*****")
    # logger.debug("Station: %s", f"{self.station}")
    # logger.debug("Contest: %s", f"{self.contest_settings}")
    now = datetime.datetime.now()
    date_time = now.strftime("%Y-%m-%d_%H-%M-%S")
    filename = (
        str(Path.home())
        + "/"
        + f"{self.station.get('Call', '').upper()}_{cabrillo_name}_{date_time}.log"
    )
    # logger.debug("%s", filename)
    log = self.database.fetch_all_contacts_asc()
    try:
        with open(filename, "w", encoding=file_encoding, newline="") as file_descriptor:
            output_cabrillo_line(
                "START-OF-LOG: 3.0",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CREATED-BY: Not1MM v{__version__}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CONTEST: {cabrillo_name}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            if self.station.get("Club", ""):
                output_cabrillo_line(
                    f"CLUB: {self.station.get('Club', '').upper()}",
                    "\r\n",
                    file_descriptor,
                    file_encoding,
                )
            output_cabrillo_line(
                f"CALLSIGN: {self.station.get('Call','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"LOCATION: {self.station.get('ARRLSection', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-OPERATOR: {self.contest_settings.get('OperatorCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-ASSISTED: {self.contest_settings.get('AssistedCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-BAND: {self.contest_settings.get('BandCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            mode = self.contest_settings.get("ModeCategory", "")
            if mode in ["SSB+CW", "SSB+CW+DIGITAL"]:
                mode = "MIXED"
            output_cabrillo_line(
                f"CATEGORY-MODE: {mode}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-TRANSMITTER: {self.contest_settings.get('TransmitterCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            if self.contest_settings.get("OverlayCategory", "") != "N/A":
                output_cabrillo_line(
                    f"CATEGORY-OVERLAY: {self.contest_settings.get('OverlayCategory','')}",
                    "\r\n",
                    file_descriptor,
                    file_encoding,
                )
            output_cabrillo_line(
                f"GRID-LOCATOR: {self.station.get('GridSquare','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-POWER: {self.contest_settings.get('PowerCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )

            output_cabrillo_line(
                f"CLAIMED-SCORE: {calc_score(self)}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            ops = f"@{self.station.get('Call','')}"
            list_of_ops = self.database.get_ops()
            for op in list_of_ops:
                ops += f", {op.get('Operator', '')}"
            output_cabrillo_line(
                f"OPERATORS: {ops}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"NAME: {self.station.get('Name', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS: {self.station.get('Street1', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-CITY: {self.station.get('City', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-STATE-PROVINCE: {self.station.get('State', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-POSTALCODE: {self.station.get('Zip', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-COUNTRY: {self.station.get('Country', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"EMAIL: {self.station.get('Email', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            for contact in log:
                the_date_and_time = contact.get("TS", "")
                themode = contact.get("Mode", "")
                if themode == "LSB" or themode == "USB":
                    themode = "PH"
                frequency = str(int(contact.get("Freq", "0"))).rjust(5)

                loggeddate = the_date_and_time[:10]
                loggedtime = the_date_and_time[11:13] + the_date_and_time[14:16]
                output_cabrillo_line(
                    f"QSO: {frequency} {themode} {loggeddate} {loggedtime} "
                    f"{contact.get('StationPrefix', '').ljust(13)} "
                    f"{str(contact.get('SNT', '')).ljust(3)} "
                    f"{str(contact.get('SentNr', '')).ljust(6)} "
                    f"{contact.get('Call', '').ljust(13)} "
                    f"{str(contact.get('RCV', '')).ljust(3)} "
                    f"{str(contact.get('NR', '')).ljust(6)}",
                    "\r\n",
                    file_descriptor,
                    file_encoding,
                )
            output_cabrillo_line("END-OF-LOG:", "\r\n", file_descriptor, file_encoding)
        self.show_message_box(f"Cabrillo saved to: {filename}")
    except IOError as exception:
        # logger.critical("cabrillo: IO error: %s, writing to %s", exception, filename)
        self.show_message_box(f"Error saving Cabrillo: {exception} {filename}")
        return


def recalculate_mults(self):
    """Recalculates multipliers after change in logged qso."""


def get_mults(self):
    """"""

    mults = {}
    return mults


def just_points(self):
    """"""
    return get_points(self)



"""General Logging plugin"""

# pylint: disable=invalid-name, unused-argument, unused-variable, c-extension-no-member, unused-import


# from not1mm.lib.ham_utility import get_logged_band
from renfield.lib.plugin_common import gen_adif, get_points

# from not1mm.lib.version import __version__

name = "General Logging"
cabrillo_name = "General-Logging"
mode = "BOTH"  # CW SSB BOTH RTTY


dupe_type = 4


def points(self):
    """Calc point"""


def show_mults(self):
    """Return display string for mults"""


def show_qso(self):
    """Return qso count"""
    result = self.database.fetch_qso_count()
    if result:
        return int(result.get("qsos", 0))
    return 0


def calc_score(self):
    """Return calculated score"""
    # result = self.database.fetch_points()


def adif(self):
    """Call the generate ADIF function"""
    gen_adif(self, cabrillo_name)


def cabrillo(self, file_encoding):
    """Generates Cabrillo file. Maybe."""


def recalculate_mults(self):
    """Recalculates multipliers after change in logged qso."""


def check_call_history(self):
    """"""
    result = self.database.fetch_call_history(self.callsign.text())
    if result:
        self.history_info.setText(f"{result.get('UserText','')}")
        if self.other_1.text() == "":
            self.other_1.setText(f"{result.get('Name', '')}")



#!/usr/bin/env python3
"""Not1MM Contest Contact Aggregation Server"""

import logging
import importlib
import socket
import time

from time import gmtime, strftime

import datetime

# import os
import sys

# import threading
# import argparse
import queue

# import pkgutil
# from itertools import chain
from json import JSONDecodeError, loads, dumps

# from pathlib import Path
# from shutil import copyfile
# Import path may change depending on if it's dev or production.
try:
    import renfield.lib.fsutils as fsutils
    from renfield.lib.version import __version__
    from renfield.lib.database import DataBase
except (ImportError, ModuleNotFoundError):
    import lib.fsutils as fsutils
    from lib.version import __version__
    from lib.database import DataBase

from rich.text import Text
from textual.app import App, ComposeResult
from textual.widgets import Static, Header, Footer, Placeholder, DataTable
from textual.containers import VerticalScroll, Horizontal, Vertical, Grid, Container

# from textual import log


# pylint: disable=no-name-in-module, invalid-name, c-extension-no-member, global-statement


class Trafficlog:
    """holds recent udp log traffic"""

    def __init__(self):
        self.items = []

    def add_item(self, item):
        """adds an item to the log and trims the log"""
        self.items.append(item)
        if len(self.items) > 16:
            self.items = self.items[1 : len(self.items)]

    def get_log(self):
        """returns a list of log items"""
        return self.items


def doimp(modname) -> object:
    """
    Imports a module.

    Parameters
    ----------
    modname : str
    The name of the module to import.

    Returns
    -------
    object
    The module object.
    """

    # logger.debug("doimp: %s", modname)
    try:
        return importlib.import_module(f"plugins.{modname}")
    except (ImportError, ModuleNotFoundError):
        return importlib.import_module(f"renfield.plugins.{modname}")


# def list_contests():
#     """Prints out an enumerated list of contests"""

#     for index, item in enumerate(plugins, start=1):
#         print(f"{index}: {item}")


class Msg(Static):
    """A widget to display messages."""

    def __init__(self) -> None:
        super().__init__()
        self.contents = [""]

    def on_mount(self) -> None:
        ...
        # self.update("Hello from renfield!")

    def on_update(self, message: str) -> None:
        """Update the message."""
        self.contents.insert(0, message)
        message = "\n".join(self.contents)
        self.update(message)


class NetworkInfo(DataTable):
    """A widget to display UPD network information."""

    def __init__(self) -> None:
        super().__init__()

    def on_mount(self) -> None:
        ...
        # self.update("Hello from renfield!")

    def on_update(self, group: str, port: int, interface: str) -> None:
        """Update the message."""
        ROWS = [
            ("", ""),
            (Text("Group:", justify="right"), f"{group}"),
            (Text("Port:", justify="right"), f"{port}"),
            (Text("Interface:", justify="right"), f"{interface}"),
        ]
        self.show_header = False
        self.show_cursor = False
        self.add_columns(*ROWS[0])
        self.add_rows(ROWS[1:])


class ContestInfo(DataTable):
    """A widget to display the current contest information."""

    def __init__(self) -> None:
        super().__init__()

    def on_mount(self) -> None:
        ...
        # self.update("Hello from renfield!")

    def on_update(self, station: str, contest: str, interface: str) -> None:
        """Update the message."""
        ROWS = [
            ("", ""),
            (Text("Station:", justify="right"), f"{station}"),
            (Text("Contest:", justify="right"), f"{contest}"),
            # (Text("Interface:", justify="right"), f"{interface}"),
        ]
        self.show_header = False
        self.show_cursor = False
        self.clear()
        self.add_columns(*ROWS[0])
        self.add_rows(ROWS[1:])


class ContactsInfo(DataTable):
    """A widget to display a table of contact counts by band and mode."""

    def __init__(self) -> None:
        super().__init__()

    def on_mount(self) -> None:
        self.zebra_style = True
        self.show_header = True
        self.show_cursor = False
        ROWS = [("Band", "QSO", "CW", "PH", "DI", "Pts")]
        self.add_columns(*ROWS[0])

    def on_update(self, stats: list[tuple[str, str, str, str, str, str]]) -> None:
        """Update the message."""
        self.clear()
        self.add_rows(stats)


class OperatorInfo(DataTable):
    """A widget to display contest operators."""

    def __init__(self) -> None:
        super().__init__()

    def on_mount(self) -> None:
        self.zebra_style = True
        self.show_header = True
        self.show_cursor = False
        ROWS = [("Operator", "Log Pos", "Band", "Mode")]
        self.add_columns(*ROWS[0])

    def on_update(self, op_list) -> None:
        """Update the widget."""
        ops = []
        for opname in op_list.keys():
            items = op_list.get(opname, ["", "", ""])
            ops.append([opname, items[0], items[1], items[2]])
        self.clear()
        self.add_rows(ops)


class Application(App):

    BINDINGS = [
        ("q", "quit_app", "Exit the application"),
        ("R", "reset_db", "Reset the database"),
    ]
    contest = None

    def __init__(self):
        super().__init__()
        self.DB: DataBase = DataBase("server_database.db", fsutils.HOME_PATH)
        self.active_contest: str | None = None
        self.station: str | None = None
        self.contest: object | None = None
        self.operators_seen = {}
        self.udp_fifo: queue.Queue = queue.Queue()
        self.MULTICAST_PORT: int = 2239
        self.MULTICAST_GROUP: str = "224.1.1.1"
        self.INTERFACE_IP: str = "0.0.0.0"
        self.network_socket: socket.socket = socket.socket(
            socket.AF_INET, socket.SOCK_DGRAM
        )
        if sys.platform.startswith("darwin"):
            self.network_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
        else:
            self.network_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

        self.network_socket.bind(("", self.MULTICAST_PORT))
        mreq = socket.inet_aton(self.MULTICAST_GROUP) + socket.inet_aton(
            self.INTERFACE_IP
        )
        self.network_socket.setsockopt(
            socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, bytes(mreq)
        )
        self.network_socket.settimeout(0.1)

    def compose(self) -> ComposeResult:
        """Create child widgets for the app."""
        self.server_msg = Msg()
        self.networkinfo = NetworkInfo()
        self.contestinfo = ContestInfo()
        self.contactsinfo = ContactsInfo()
        self.operatorinfo = OperatorInfo()

        yield Header()
        with Vertical():
            with Horizontal(id="h1"):
                yield Container(self.contestinfo, id="contestinfo")
                yield Container(self.networkinfo, id="networkinfo")
                yield Placeholder(id="ph5")
            with Horizontal(id="h2"):
                yield Vertical(VerticalScroll(self.server_msg, id="scroll"), id="v1")
                yield Vertical(
                    Container(self.contactsinfo, id="ph1"),
                    Container(self.operatorinfo, id="ph2"),
                    id="v2",
                )
        yield Footer()

    def on_mount(self) -> None:
        """Called when widget is first added."""

        self.title = "renfield"
        self.sub_title = f"v{__version__}"

        self.server_msg.styles.text_overflow = "ellipsis"
        self.networkinfo.styles.text_overflow = "ellipsis"
        self.networkinfo.styles.align = ("center", "middle")
        # self.server_msg.styles.width = "2fr"

        widget = self.query_one("#scroll")
        widget.styles.overflow_y = "scroll"
        widget.styles.border = ("solid", "green")
        widget.styles.width = "100%"
        widget.border_title = "[blue]UDP Traffic[/]"

        h1 = self.query_one("#h1")
        h2 = self.query_one("#h2")
        h1.styles.height = "1fr"
        h2.styles.height = "2fr"
        contestinfo = self.query_one("#contestinfo")
        networkinfo = self.query_one("#networkinfo")
        ph5 = self.query_one("#ph5")
        contestinfo.styles.width = "1fr"
        networkinfo.styles.width = "1fr"
        ph5.styles.width = "1fr"
        contestinfo.styles.border = ("solid", "green")
        networkinfo.styles.border = ("solid", "green")
        ph5.styles.border = ("solid", "green")
        contestinfo.border_title = "[blue]Group[/]"
        networkinfo.border_title = "[blue]Network[/]"
        ph5.border_title = "[blue]Scoring[/]"

        v1 = self.query_one("#v1")
        v2 = self.query_one("#v2")
        v1.styles.width = "2fr"
        v2.styles.width = "1fr"

        ph1 = self.query_one("#ph1")
        ph1.styles.height = "1fr"
        ph1.styles.border = ("solid", "green")
        ph1.border_title = "[blue]Contacts[/]"

        ph2 = self.query_one("#ph2")
        ph2.styles.height = "1fr"
        ph2.styles.border = ("solid", "green")
        ph2.border_title = "[blue]Operators[/]"

        self.set_interval(0.2, self.server_message)
        self.set_interval(10.0, self.send_pulse)
        self.update_network_window()
        self.update_contest_window()
        self.update_contacts_window()

    def action_quit_app(self) -> None:
        """Exit the application."""
        self.exit()

    def action_reset_db(self) -> None:
        """Reset the contest database."""
        self.DB.reset_database()
        self.update_contacts_window()

    def send_pulse(self) -> None:
        """send heartbeat"""
        try:
            pulse = b'{"cmd": "PING", "host": "server"}'
            self.network_socket.sendto(
                pulse, (self.MULTICAST_GROUP, self.MULTICAST_PORT)
            )
        except OSError as error:
            logging.critical(f"OSError: {error}")

        if self.active_contest is None:
            self.send_contest_request()

    def send_contest_request(self) -> None:
        """send heartbeat"""
        try:
            pulse = b'{"cmd": "CONTEST_REQUEST", "host": "server"}'
            self.network_socket.sendto(
                pulse, (self.MULTICAST_GROUP, self.MULTICAST_PORT)
            )
        except OSError as error:
            logging.critical(f"OSError: {error}")

    def server_message(self) -> None:
        """Respond to network messages"""

        timestamp = strftime("%H:%M:%S", gmtime())
        try:
            payload = self.network_socket.recv(2048)
        except socket.timeout:
            return
        if not payload:
            return
        try:
            json_data = loads(payload.decode())
        except UnicodeDecodeError as err:
            the_error = f"Not Unicode: {err}\n{payload}\n"
            print(the_error)
            return
        except JSONDecodeError as err:
            the_error = f"Not JSON: {err}\n{payload}\n"
            print(the_error)
            return
        print(json_data)

        if json_data.get("cmd") == "POST":
            """
            {
                'TS': '2025-05-05 20:44:46',
                'Call': 'K5TUX',
                'Freq': 14030.0,
                'QSXFreq': 14030.0,
                'Mode': 'CW',
                'ContestName': 'ARRL-FIELD-DAY',
                'SNT': '599',
                'RCV': '599',
                'CountryPrefix': 'K',
                'StationPrefix': 'K6GTE',
                'QTH': '',
                'Name': '',
                'Comment': '',
                'NR': 0,
                'Sect': 'MO',
                'Prec': '',
                'CK': 0,
                'ZN': 4,
                'SentNr': 0,
                'Points': 2,
                'IsMultiplier1': 0,
                'IsMultiplier2': 0,
                'Power': 0,
                'Band': '14',
                'WPXPrefix': 'K5',
                'Exchange1': '1B',
                'RadioNR': '',
                'ContestNR': '4',
                'isMultiplier3': 0,
                'MiscText': '',
                'IsRunQSO': False,
                'ContactType': '',
                'Run1Run2': '',
                'GridSquare': '',
                'Operator': 'K6GTE',
                'Continent': 'NA',
                'RoverLocation': '',
                'RadioInterfaced': '',
                'NetworkedCompNr': 1,
                'NetBiosName': 'fredo',
                'IsOriginal': 1,
                'ID': '42a30a19379b4171a810bb6b8ac4d9ce',
                'CLAIMEDQSO': 1,
                'cmd': 'POST',
                'expire': '2025-05-05T13:45:16.061513'
            }
            """
            self.server_msg.on_update(
                f"\\[{timestamp}] CMD:{json_data.get('cmd', '')}:{json_data.get('Call', '')} From:{json_data.get('NetBiosName', '')}:{json_data.get('Operator', '')} "
            )
            json_data.pop("cmd", None)
            json_data.pop("expire", None)

            self.DB.log_contact(json_data)

            packet = {"cmd": "RESPONSE"}
            packet["recipient"] = json_data.get("NetBiosName")
            packet["subject"] = "POST"
            packet["unique_id"] = json_data.get("ID")
            sendme = bytes(dumps(packet), encoding="ascii")
            self.network_socket.sendto(
                sendme, (self.MULTICAST_GROUP, self.MULTICAST_PORT)
            )
            self.update_contacts_window()
            self.operators_seen[json_data.get("Operator", "Unknown")] = [
                json_data.get("NetBiosName", "Unknown"),
                json_data.get("Band", "Unknown"),
                json_data.get("Mode", "Unknown"),
            ]
            self.update_operators_window()
            return

        if json_data.get("cmd") == "STATION_STATE":
            self.operators_seen[json_data.get("Operator", "Unknown")] = [
                json_data.get("NetBiosName", "Unknown"),
                json_data.get("Band", "Unknown"),
                json_data.get("Mode", "Unknown"),
            ]
            self.update_operators_window()
            return

        if json_data.get("cmd") == "LOG":
            self.server_msg.on_update(f"Got {json_data.get("cmd")}: {json_data=} ")
            # LOG.add_item(f"[{timestamp}] GENERATE LOG: {json_data.get('station')}")
            # cabrillo()
            # packet = {"cmd": "RESPONSE"}
            # packet["recipient"] = json_data.get("station")
            # packet["subject"] = "LOG"
            # bytes_to_send = bytes(dumps(packet), encoding="ascii")
            # s.sendto(bytes_to_send, (MULTICAST_GROUP, MULTICAST_PORT))
            # LOG.add_item(
            #     f"[{timestamp}] GENERATE LOG CONF: {json_data.get('station')}"
            # )
            return

        if json_data.get("cmd") == "DUPE":
            self.server_msg.on_update(f"Got {json_data.get("cmd")}: {json_data=} ")
            # LOG.add_item(f"[{timestamp}] Checking Dupe: {json_data.get('contact')}")
            # packet = {"cmd": "RESPONSE"}
            # packet["recipient"] = json_data.get("station")
            # packet["subject"] = "DUPE"
            # packet["contact"] = json_data.get("contact")
            # _call = json_data.get("contact")
            # _mode = json_data.get("mode")
            # _band = json_data.get("band")
            # the_count = DB.get_dupe_status(_call, _band, _mode)
            # packet["isdupe"] = the_count
            # bytes_to_send = bytes(dumps(packet), encoding="ascii")
            # s.sendto(bytes_to_send, (MULTICAST_GROUP, MULTICAST_PORT))
            # # comm_log()
            return

        if json_data.get("cmd") == "DELETE":
            self.server_msg.on_update(
                f"\\[{timestamp}] CMD:{json_data.get('cmd', '')} From:{json_data.get('station', '')} ID:{json_data.get('ID', '')}"
            )

            self.DB.delete_contact(json_data.get("unique_id", ""))

            packet = {"cmd": "RESPONSE"}
            packet["recipient"] = json_data.get("station")
            packet["subject"] = "DELETE"
            packet["unique_id"] = json_data.get("unique_id")
            sendme = bytes(dumps(packet), encoding="ascii")
            self.network_socket.sendto(
                sendme, (self.MULTICAST_GROUP, self.MULTICAST_PORT)
            )
            self.update_contacts_window()
            return

        if json_data.get("cmd") == "CONTACTCHANGED":
            self.server_msg.on_update(
                f"\\[{timestamp}] CMD:{json_data.get('cmd', '')} From:{json_data.get('NetBiosName', '')} OP:{json_data.get('Operator', '')} ID:{json_data.get('ID', '')}"
            )
            print(json_data)
            # Construct a response.
            packet = {"cmd": "RESPONSE"}
            packet["recipient"] = json_data.get("NetBiosName")
            packet["subject"] = "CONTACTCHANGED"
            packet["unique_id"] = json_data.get("ID")

            # Strip out unneeded keys value pairs.
            json_data.pop("cmd", None)
            json_data.pop("station", None)
            json_data.pop("expire", None)
            json_data.pop("unique_id", None)

            self.DB.change_contact(json_data)
            sendme = bytes(dumps(packet), encoding="ascii")
            self.network_socket.sendto(
                sendme, (self.MULTICAST_GROUP, self.MULTICAST_PORT)
            )
            self.update_contacts_window()
            return

        if json_data.get("cmd") == "NEWDB":
            # {"ContestID": 1, "ContestName": "general_logging", "StartDate": "2025-08-13 00:00:00",
            # "OperatorCategory": "SINGLE-OP", "BandCategory": "ALL", "PowerCategory": "LOW",
            # "ModeCategory": "CW", "OverlayCategory": "N/A", "ClaimedScore": "'''''''0'''''''",
            # "Operators": "k6gte", "Soapbox": "", "SentExchange": "", "ContestNR": 1, "SubType": null,
            # "StationCategory": "FIXED", "AssistedCategory": "ASSISTED", "TransmitterCategory": "ONE",
            # "TimeCategory": null, "cmd": "NEWDB", "expire": "2025-08-24T09:36:36.212594",
            # "NetBiosName": "fredo", "Operator": "K6GTE", "ID": "713af39d088d467ab7dfc431debc59e9",
            # "Station": {"Call": "K6GTE", "Name": "Michael Bridak", "Email": "michael.bridak@gmail.com",
            # "Street1": "2854 W Bridgeport Ave", "Street2": "", "City": "Anaheim", "State": "CA",
            # "Zip": "92804", "Country": "United States", "GridSquare": "dm13at", "LicenseClass": "General",
            # "Latitude": 33.8125, "Longitude": -117.9583, "PacketNode": "N/A", "ARRLSection": "ORG",
            # "Club": "", "IARUZone": 6, "CQZone": 3, "STXeq": "", "SPowe": "", "SAnte": "", "SAntH1": "",
            # "SAntH2": "", "RoverQTH": ""}}

            self.server_msg.on_update(
                f"\\[{timestamp}] CMD:{json_data.get('cmd', '')} From:{json_data.get('NetBiosName', '')} OP:{json_data.get('Operator', '')}"
            )
            globals()["station"] = json_data.get("Station")

            self.active_contest = json_data.get("ContestName", "")
            self.contest = doimp(json_data.get("ContestName"))
            self.station = json_data.get("Station", {}).get("Call", "")
            self.update_contest_window()
            self.server_msg.on_update("")
            self.update_contacts_window()
            print(f"Active contest set to {self.active_contest}")
            packet = {"cmd": "RESPONSE"}
            packet["recipient"] = json_data.get("NetBiosName")
            packet["subject"] = "NEWDB"
            packet["unique_id"] = json_data.get("ID")
            sendme = bytes(dumps(packet), encoding="ascii")
            self.network_socket.sendto(
                sendme, (self.MULTICAST_GROUP, self.MULTICAST_PORT)
            )
            return

        if json_data.get("cmd") == "CURRENT_CONTEST":
            self.active_contest = json_data.get("ContestName", "")
            self.server_msg.on_update("")
            self.contest = doimp(self.active_contest)
            print(f"Active contest set to {self.active_contest}")

    def update_network_window(self) -> None:
        """Shows the network information."""
        self.networkinfo.on_update(
            self.MULTICAST_GROUP, self.MULTICAST_PORT, self.INTERFACE_IP
        )

    def update_contest_window(self) -> None:
        """Shows the contest information."""
        self.contestinfo.on_update(self.station, self.active_contest, "")

    def update_contacts_window(self) -> None:
        """Shows the contacts information."""
        self.contactsinfo.on_update(self.DB.get_statistics())

    def update_operators_window(self) -> None:
        """
        Shows the operators information.
        operator_list = list[tuple[str, str, str]]
        """
        self.operatorinfo.on_update(self.operators_seen)


def run():
    app = Application()
    app.run()


if __name__ == "__main__":
    run()



"""Database class to store contacts"""

import logging
import sqlite3


class DataBase:
    """Database class for our database."""

    def __init__(self, database):
        """initializes DataBase instance"""
        self.database = database
        self.create_db()

    def create_db(self) -> None:
        """
        create database tables contacts if they do not exist.
        """
        try:
            with sqlite3.connect(self.database) as conn:
                cursor = conn.cursor()
                sql_table = (
                    "CREATE TABLE IF NOT EXISTS contacts "
                    "(id INTEGER PRIMARY KEY, "
                    "unique_id text NOT NULL, "
                    "callsign text NOT NULL, "
                    "class text NOT NULL, "
                    "section text NOT NULL, "
                    "date_time text NOT NULL, "
                    "frequency INTEGER DEFAULT 0, "
                    "band text NOT NULL, "
                    "mode text NOT NULL, "
                    "power INTEGER NOT NULL, "
                    "grid text NOT NULL, "
                    "opname text NOT NULL, "
                    "station text NOT NULL);"
                )
                cursor.execute(sql_table)
                conn.commit()
        except sqlite3.Error as exception:
            logging.critical("create_db: Unable to create database: %s", exception)

    def log_contact(self, logme: tuple) -> None:
        """
        Inserts a contact into the db.
        pass in (unique_id, hiscall, hisclass, hissection, band, mode, int(power), grid, name)
        """
        try:
            with sqlite3.connect(self.database) as conn:
                sql = f"select count(*) from contacts where unique_id ='{logme[0]}';"
                cur = conn.cursor()
                cur.execute(sql)
                if cur.fetchone()[0] == 0:
                    sql = (
                        "INSERT INTO contacts"
                        "(unique_id, callsign, class, section, date_time, frequency, "
                        "band, mode, power, grid, opname, station) "
                        "VALUES(?,?,?,?,?,?,?,?,?,?,?,?)"
                    )
                    cur.execute(sql, logme)
                else:
                    sql = (
                        f"update contacts set callsign = '{logme[1]}', class = '{logme[2]}', "
                        f"section = '{logme[3]}', date_time = '{logme[4]}', band = '{logme[6]}', "
                        f"mode = '{logme[7]}', power = '{logme[8]}', station = '{logme[11]}', "
                        f"frequency = '{logme[5]}' where unique_id='{logme[0]}';"
                    )
                    cur.execute(sql)
                conn.commit()
        except sqlite3.Error as exception:
            logging.debug("DataBase log_contact: %s", exception)

    def delete_contact(self, unique_id) -> None:
        """Deletes a contact from the db by passing in UUID."""
        if unique_id:
            try:
                with sqlite3.connect(self.database) as conn:
                    sql = f"delete from contacts where unique_id='{unique_id}'"
                    cur = conn.cursor()
                    cur.execute(sql)
                    conn.commit()
            except sqlite3.Error as exception:
                logging.debug("DataBase delete_contact: %s", exception)

    def change_contact(self, qso):
        """Update an existing contact."""
        try:
            with sqlite3.connect(self.database) as conn:
                sql = (
                    f"update contacts set callsign = '{qso[0]}', class = '{qso[1]}', "
                    f"section = '{qso[2]}', date_time = '{qso[3]}', band = '{qso[4]}', "
                    f"mode = '{qso[5]}', power = '{qso[6]}', station = '{qso[7]}', "
                    f"frequency = '{qso[8]}' where unique_id='{qso[9]}';"
                )
                cur = conn.cursor()
                cur.execute(sql)
                conn.commit()
        except sqlite3.Error as exception:
            logging.critical("DataBase change_contact: %s", exception)

    def get_operators(self) -> list:
        """Return a list of station calls used."""
        with sqlite3.connect(self.database) as conn:
            cursor = conn.cursor()
            cursor.execute("select distinct station from contacts;")
            ops = cursor.fetchall()
        return ops

    def stats(self) -> tuple:
        """
        returns a tuple with some stats:
        cwcontacts, phonecontacts, digitalcontacts, bandmodemult, last15, lasthour, hignpower, qrp
        """
        with sqlite3.connect(self.database) as conn:
            cursor = conn.cursor()
            cursor.execute("select count(*) from contacts where mode = 'CW'")
            cwcontacts = str(cursor.fetchone()[0])
            cursor.execute("select count(*) from contacts where mode = 'PH'")
            phonecontacts = str(cursor.fetchone()[0])
            cursor.execute("select count(*) from contacts where mode = 'DI'")
            digitalcontacts = str(cursor.fetchone()[0])
            cursor.execute("select distinct band, mode from contacts")
            bandmodemult = len(cursor.fetchall())
            cursor.execute(
                "SELECT count(*) FROM contacts "
                "where datetime(date_time) >=datetime('now', '-15 Minutes')"
            )
            last15 = str(cursor.fetchone()[0])
            cursor.execute(
                "SELECT count(*) FROM contacts "
                "where datetime(date_time) >=datetime('now', '-1 Hours')"
            )
            lasthour = str(cursor.fetchone()[0])
            cursor.execute(
                "select count(*) as qrpc from contacts where mode = 'CW' and power > 5"
            )
            log = cursor.fetchall()
            qrpc = list(log[0])[0]
            cursor.execute(
                "select count(*) as qrpp from contacts where mode = 'PH' and power > 10"
            )
            log = cursor.fetchall()
            qrpp = list(log[0])[0]
            cursor.execute(
                "select count(*) as qrpd from contacts where mode = 'DI' and power > 10"
            )
            log = cursor.fetchall()
            qrpd = list(log[0])[0]
            cursor.execute(
                "select count(*) as highpower from contacts where power > 100"
            )
            log = cursor.fetchall()
            highpower = bool(list(log[0])[0])
            qrp = not qrpc + qrpp + qrpd

            return (
                cwcontacts,
                phonecontacts,
                digitalcontacts,
                bandmodemult,
                last15,
                lasthour,
                highpower,
                qrp,
            )

    def contacts_under_101watts(self) -> tuple:
        """return contact tallies for contacts made below 101 watts."""
        try:
            with sqlite3.connect(self.database) as conn:
                cursor = conn.cursor()
                cursor.execute(
                    "select count(*) as cw from contacts where mode = 'CW' and power < 101"
                )
                c_dubs = str(cursor.fetchone()[0])
                cursor.execute(
                    "select count(*) as ph from contacts where mode = 'PH' and power < 101"
                )
                phone = str(cursor.fetchone()[0])
                cursor.execute(
                    "select count(*) as di from contacts where mode = 'DI' and power < 101"
                )
                digital = str(cursor.fetchone()[0])
        except sqlite3.Error as exception:
            logging.critical("DB-contacts_under_101watts: %s", exception)
            return 0, 0, 0
        return c_dubs, phone, digital

    def qrp_check(self) -> tuple:
        """check to see if all contacts were QRP"""
        try:
            with sqlite3.connect(self.database) as conn:
                cursor = conn.cursor()
                cursor.execute(
                    "select count(*) as qrpc from contacts where mode = 'CW' and power > 5"
                )
                log = cursor.fetchall()
                qrpc = list(log[0])[0]
                cursor.execute(
                    "select count(*) as qrpp from contacts where mode = 'PH' and power > 10"
                )
                log = cursor.fetchall()
                qrpp = list(log[0])[0]
                cursor.execute(
                    "select count(*) as qrpd from contacts where mode = 'DI' and power > 10"
                )
                log = cursor.fetchall()
                qrpd = list(log[0])[0]
                cursor.execute(
                    "select count(*) as highpower from contacts where power > 100"
                )
                log = cursor.fetchall()
                highpower = bool(list(log[0])[0])
                qrp = not qrpc + qrpp + qrpd
        except sqlite3.Error as exception:
            logging.critical("qrpcheck: %s", exception)
            return 0, 0
        return qrp, highpower

    def get_band_mode_tally(self, band, mode):
        """
        returns the amount of contacts and the maximum power used
        for a given band using a particular mode.
        """
        with sqlite3.connect(self.database) as conn:
            cursor = conn.cursor()
            cursor.execute(
                "select count(*) as tally, MAX(power) as mpow from contacts "
                f"where band = '{band}' AND mode ='{mode}'"
            )
            return cursor.fetchone()

    def get_bands(self) -> tuple:
        """returns a list of bands"""
        with sqlite3.connect(self.database) as conn:
            cursor = conn.cursor()
            cursor.execute("select DISTINCT band from contacts")
            return cursor.fetchall()

    def fetch_all_contacts_asc(self) -> tuple:
        """returns a tuple of all contacts in the database."""
        with sqlite3.connect(self.database) as conn:
            cursor = conn.cursor()
            cursor.execute("select * from contacts order by date_time ASC")
            return cursor.fetchall()

    def fetch_all_contacts_desc(self) -> tuple:
        """returns a tuple of all contacts in the database."""
        with sqlite3.connect(self.database) as conn:
            cursor = conn.cursor()
            cursor.execute("select * from contacts order by date_time desc")
            return cursor.fetchall()

    def fetch_last_contact(self) -> tuple:
        """returns a tuple of all contacts in the database."""
        with sqlite3.connect(self.database) as conn:
            cursor = conn.cursor()
            cursor.execute("select * from contacts order by id desc")
            return cursor.fetchone()

    def dup_check(self, acall: str) -> tuple:
        """returns a list of possible dups"""
        with sqlite3.connect(self.database) as conn:
            cursor = conn.cursor()
            cursor.execute(
                "select callsign, class, section, band, mode "
                f"from contacts where callsign like '{acall}' order by band"
            )
            return cursor.fetchall()

    def sections(self) -> tuple:
        """returns a list of sections worked."""
        with sqlite3.connect(self.database) as conn:
            cursor = conn.cursor()
            cursor.execute("select distinct section from contacts")
            return cursor.fetchall()

    def contact_by_id(self, record) -> tuple:
        """returns a contact matching an id"""
        with sqlite3.connect(self.database) as conn:
            cursor = conn.cursor()
            cursor.execute("select * from contacts where id=" + record)
            return cursor.fetchall()

    def get_grids(self) -> tuple:
        """returns a tuple of unique grids in the log."""
        with sqlite3.connect(self.database) as conn:
            cursor = conn.cursor()
            cursor.execute("select DISTINCT grid from contacts")
            return cursor.fetchall()

    def get_dupe_status(self, call, band, mode):
        """See is call exists on band/mode return bool"""
        with sqlite3.connect(self.database) as conn:
            cursor = conn.cursor()
            cursor.execute(
                f"select count(*) as dupecount from contacts where "
                f"callsign = '{call}' AND band = '{band}' AND mode = '{mode}';"
            )
            return cursor.fetchone()[0]



"""
K6GTE, Database class to store contacts
Email: michael.bridak@gmail.com
GPL V3
"""

# pylint: disable=line-too-long

# get Saturday plus 48 hours: select datetime('now', 'WEEKDAY 6','48 HOURS');
# DROP TABLE IF EXISTS t1;

import os
import sqlite3

if __name__ == "__main__":
    print("I'm not the program you are looking for.")


class DataBase:
    """Database class for our database."""

    current_contest = 1

    def __init__(self, database: str, app_data_dir: str):
        """initializes DataBase instance"""
        print(f"Database: {database}")
        self.app_data_dir = app_data_dir

        self.database = app_data_dir + "/" + database
        self.create_dxlog_table()

    def reset_database(self):
        """Reset DataBase instance"""
        print(f"Resetting database: {self.database}")
        try:
            os.remove(self.database)
        except OSError:
            ...
        self.create_dxlog_table()

    @staticmethod
    def row_factory(cursor, row):
        """
        cursor.description:
        (name, type_code, display_size,
        internal_size, precision, scale, null_ok)
        row: (value, value, ...)
        """
        return {
            col[0]: row[idx]
            for idx, col in enumerate(
                cursor.description,
            )
        }

    def create_dxlog_table(self) -> None:
        """creates the dxlog table"""
        print("Creating DXLOG Table")
        try:
            with sqlite3.connect(self.database) as conn:
                cursor = conn.cursor()
                sql_command = (
                    "CREATE TABLE IF NOT EXISTS DXLOG ("
                    "TS DATETIME NOT NULL, "
                    "Call VARCHAR(15) NOT NULL, "
                    "Freq DOUBLE NULL, "
                    "QSXFreq DOUBLE NULL DEFAULT 0, "
                    "Mode VARCHAR(6) DEFAULT '', "
                    "ContestName VARCHAR(10) DEFAULT 'NORMAL', "
                    "SNT VARCHAR(10) DEFAULT '', "
                    "RCV VARCHAR(15) DEFAULT '', "
                    "CountryPrefix VARCHAR(8) DEFAULT '', "
                    "StationPrefix VARCHAR(15) DEFAULT '', "
                    "QTH VARCHAR(25) DEFAULT '', "
                    "Name VARCHAR(20) DEFAULT '', "
                    "Comment VARCHAR(60) DEFAULT '', "
                    "NR INTEGER DEFAULT 0, "
                    "Sect VARCHAR(8) DEFAULT '', "
                    "Prec VARCHAR(1) DEFAULT '', "
                    "CK TINYINT DEFAULT 0, "
                    "ZN TINYINT DEFAULT 0, "
                    "SentNr INTEGER DEFAULT 0, "
                    "Points INTEGER DEFAULT 0, "
                    "IsMultiplier1 TINYINT DEFAULT 0, "
                    "IsMultiplier2 INTEGER DEFAULT 0, "
                    "Power VARCHAR(8) DEFAULT '', "
                    "Band FLOAT NULL DEFAULT 0, "
                    "WPXPrefix VARCHAR(8) DEFAULT '', "
                    "Exchange1 VARCHAR(20) DEFAULT '', "
                    "RadioNR TINYINT DEFAULT 1, "
                    "ContestNR INTEGER, "
                    "isMultiplier3 INTEGER DEFAULT 0, "
                    "MiscText VARCHAR(20) DEFAULT '', "
                    "IsRunQSO TINYINT(1) DEFAULT 0, "
                    "ContactType VARCHAR(1) DEFAULT '', "
                    "Run1Run2 TINYINT NOT NULL, "
                    "GridSquare VARCHAR(6) DEFAULT '', "
                    "Operator VARCHAR(20) DEFAULT '', "
                    "Continent VARCHAR(2) DEFAULT '', "
                    "RoverLocation VARCHAR(10) DEFAULT '', "
                    "RadioInterfaced INTEGER, "
                    "NetworkedCompNr INTEGER, NetBiosName varchar (255), "
                    "IsOriginal Boolean, "
                    "ID TEXT(32) NOT NULL DEFAULT '00000000000000000000000000000000', "
                    "CLAIMEDQSO INTEGER DEFAULT 1,"
                    "Dirty INTEGER DEFAULT 1,"
                    "PRIMARY KEY (`TS`, `Call`) );"
                )
                cursor.execute(sql_command)
                conn.commit()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")

    def log_contact(self, contact: dict) -> None:
        """
        Inserts a contact into the db.
        pass in a dict object, see get_empty() for keys
        """

        try:
            with sqlite3.connect(self.database) as conn:
                sql = f"SELECT count(*) from DXLOG where ID ='{contact.get('ID','')}';"
                cur = conn.cursor()
                cur.execute(sql)
                if cur.fetchone()[0] == 0:
                    pre = "INSERT INTO DXLOG("
                    values = []
                    columns = ""
                    placeholders = ""
                    for key in contact.keys():
                        columns += f"{key},"
                        values.append(contact[key])
                        placeholders += "?,"
                    post = f") VALUES({placeholders[:-1]});"
                    sql = f"{pre}{columns[:-1]}{post}"

                    cur.execute(sql, tuple(values))
                    conn.commit()
                else:
                    self.change_contact(contact)
        except sqlite3.Error as exception:
            print(f"DataBase log_contact: {exception}")

    def change_contact(self, qso: dict) -> None:
        """Update an existing contact."""

        pre = "UPDATE dxlog set "
        for key in qso.keys():
            pre += f"{key} = '{qso[key]}',"
        sql = f"{pre[:-1]} where ID='{qso['ID']}';"

        try:
            with sqlite3.connect(self.database) as conn:
                # logger.debug("%s\n%s", sql, qso)
                cur = conn.cursor()
                cur.execute(sql)
                conn.commit()
        except sqlite3.Error as exception:
            print(f"DataBase change_contact: {exception}")

    def delete_contact(self, unique_id: str) -> None:
        """Deletes a contact from the db."""
        if unique_id:
            try:
                with sqlite3.connect(self.database) as conn:
                    sql = f"delete from dxlog where ID='{unique_id}';"
                    cur = conn.cursor()
                    cur.execute(sql)
                    conn.commit()
            except sqlite3.Error as exception:
                print(f"DataBase delete_contact: {exception}")

    def fetch_all_contacts_asc(self) -> list:
        """returns a list of dicts with contacts in the database."""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select * from dxlog where ContestNR = {self.current_contest} order by TS ASC;"
                )
                return cursor.fetchall()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return ()

    def fetch_all_contacts_desc(self) -> list:
        """returns a list of dicts with contacts in the database."""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select * from dxlog where ContestNR = {self.current_contest} order by ts desc;"
                )
                return cursor.fetchall()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return ()

    def fetch_last_contact(self) -> dict:
        """returns a list of dicts with last contact in the database."""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute("select * from dxlog order by ts desc;")
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_contact_by_uuid(self, uuid: str) -> dict:
        """returns a list of dicts with last contact in the database."""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(f"select * from dxlog where ID='{uuid}';")
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_cqzn_exists(self, number) -> dict:
        """returns a dict key of nr_count"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as zn_count from dxlog where ZN = '{number}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_zn_band_count(self) -> dict:
        """
        returns dict with count of unique ZN and Band.
        {nr_count: count}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(DISTINCT(ZN || ':' || Band)) as zb_count from dxlog where ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_zn_band_mode_count(self) -> dict:
        """
        returns dict with count of unique ZN, Band and Mode.
        {nr_count: count}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(DISTINCT(ZN || ':' || Band || ':' || Mode)) as zbm_count from dxlog where ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_country_band_count(self) -> dict:
        """
        returns dict with count of unique NR.
        {cb_count: count}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(DISTINCT(CountryPrefix || ':' || Band)) as cb_count from dxlog where ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_country_count(self) -> dict:
        """
        Fetch count of unique countries
        {dxcc_count: count}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(DISTINCT(CountryPrefix)) as dxcc_count from dxlog where ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_exchange1_unique_count(self) -> dict:
        """
        Fetch count of unique countries
        {exch1_count: count}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(DISTINCT(Exchange1)) as exch1_count from dxlog where Exchange1 != '' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_arrldx_country_band_count(self) -> dict:
        """
        returns dict with count of unique NR.
        {nr_count: count}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(DISTINCT(CountryPrefix || ':' || Band)) as cb_count from dxlog where ContestNR = {self.current_contest} and points = 3;"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_arrldx_state_prov_count(self) -> dict:
        """
        returns dict with count of unique NR.
        {nr_count: count}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(DISTINCT(NR || ':' || Band)) as cb_count from dxlog where ContestNR = {self.current_contest} and points = 3;"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_nr_count(self) -> dict:
        """
        returns dict with count of unique NR.
        {nr_count: count}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(DISTINCT NR) as nr_count from dxlog where ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_nr_exists(self, number) -> dict:
        """returns a dict key of nr_count"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as nr_count from dxlog where NR = '{number}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_call_exists(self, call: str) -> dict:
        """returns a dict key of call_count"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as call_count from dxlog where Call = '{call}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_nr_exists_before_me(self, number, time_stamp) -> dict:
        """returns a dict key of nr_count"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as nr_count from dxlog where  TS < '{time_stamp}' and NR = '{number}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_call_count(self) -> dict:
        """
        returns dict with count of unique calls.
        {call_count: count}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(DISTINCT Call) as call_count from dxlog where ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_wpx_count(self) -> dict:
        """
        returns dict with count of unique WPXPrefix.
        {wpx_count: count}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(DISTINCT WPXPrefix) as wpx_count from dxlog where ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_dxcc_exists(self, dxcc) -> dict:
        """returns the dict dxcc_count of dxcc existing in current contest."""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as dxcc_count from dxlog where CountryPrefix = '{dxcc}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_dxcc_exists_before_me(self, dxcc, time_stamp) -> dict:
        """returns the dict dxcc_count of dxcc existing in current contest."""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as dxcc_count from dxlog where TS < '{time_stamp}' and CountryPrefix = '{dxcc}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_wpx_exists(self, wpx) -> dict:
        """returns a dict key of wpx_count"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as wpx_count from dxlog where WPXPrefix = '{wpx}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_wpx_exists_before_me(self, wpx, time_stamp) -> dict:
        """returns a dict key of wpx_count"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as wpx_count from dxlog where  TS < '{time_stamp}' and WPXPrefix = '{wpx}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_sect_band_exists(self, sect, band) -> dict:
        """returns a dict key of sect_count"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as sect_count from dxlog where Sect = '{sect}' and Band = '{band}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_sect_exists(self, sect) -> dict:
        """returns a dict key of sect_count"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as sect_count from dxlog where Sect = '{sect}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_sect_exists_before_me(self, sec, time_stamp) -> dict:
        """returns a dict key of sect_count"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as sect_count from dxlog where  TS < '{time_stamp}' and Sect = '{sec}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_section_band_count(self) -> dict:
        """
        returns dict with count of unique Section/Band.
        {sb_count: count}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(DISTINCT(Sect || ':' || Band)) as sb_count from dxlog where ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_section_band_count_nodx(self) -> dict:
        """
        returns dict with count of unique Section/Band.
        {sb_count: count}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    "select count(DISTINCT(Sect || ':' || Band)) as sb_count from dxlog "
                    f"where ContestNR = {self.current_contest} and Sect != 'DX';"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def check_dupe_on_band_mode(self, call, band, mode) -> dict:
        """Checks if a call is dupe on band/mode"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as isdupe from dxlog where Call = '{call}' and Mode = '{mode}' and Band = '{band}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def check_dupe_on_band(self, call, band) -> dict:
        """Checks if a call is dupe on band/mode"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as isdupe from dxlog where Call = '{call}' and Band = '{band}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def check_dupe(self, call) -> dict:
        """Checks if a call is dupe on band/mode"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as isdupe from dxlog where Call = '{call}' and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_points(self) -> dict:
        """return points"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select sum(Points) as Points from dxlog where ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_mult_count(self, mult: int) -> dict:
        """return QSO count"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as count from dxlog where IsMultiplier{mult} = 1 and ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_qso_count(self) -> dict:
        """return QSO count"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as qsos from dxlog where ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def fetch_like_calls(self, call: str) -> list:
        """returns a list of dicts with contacts in the database."""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select * from dxlog where call like '%{call}%' and ContestNR = {self.current_contest} order by TS ASC;"
                )
                return cursor.fetchall()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return ()

    def get_serial(self) -> dict:
        """Return next serial number"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select max(SentNR) + 1 as serial_nr from DXLOG where ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def get_calls_and_bands(self) -> dict:
        """
        Returns a dict like:
        {'K5TUX': [14.0, 21.0], 'N2CQR': [14.0], 'NE4RD': [14.0]}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select call, band from DXLOG where ContestNR = {self.current_contest};"
                )
                result = cursor.fetchall()
                worked_list = {}
                # This converts a list of dicts like:
                # [
                #     {"Call": "K5TUX", "Band": 14.0},
                #     {"Call": "K5TUX", "Band": 21.0},
                #     {"Call": "N2CQR", "Band": 14.0},
                #     {"Call": "NE4RD", "Band": 14.0},
                # ]
                #
                # To:
                # {'K5TUX': [14.0, 21.0], 'N2CQR': [14.0], 'NE4RD': [14.0]}
                for worked_dict in result:
                    call = worked_dict.get("Call")
                    if call in worked_list:
                        bandlist = worked_list[call]
                        bandlist.append(worked_dict["Band"])
                        worked_list[call] = bandlist
                        continue
                    worked_list[call] = [worked_dict["Band"]]
                return worked_list
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def get_like_calls_and_bands(self, call: str) -> dict:
        """
        Returns a dict like:
        {'K5TUX': [14.0, 21.0], 'N2CQR': [14.0], 'NE4RD': [14.0]}
        """
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select call, band from DXLOG where call like '%{call}%' and ContestNR = {self.current_contest};"
                )
                result = cursor.fetchall()
                worked_list = {}
                # This converts a list of dicts like:
                # [
                #     {"Call": "K5TUX", "Band": 14.0},
                #     {"Call": "K5TUX", "Band": 21.0},
                #     {"Call": "N2CQR", "Band": 14.0},
                #     {"Call": "NE4RD", "Band": 14.0},
                # ]
                #
                # To:
                # {'K5TUX': [14.0, 21.0], 'N2CQR': [14.0], 'NE4RD': [14.0]}
                for worked_dict in result:
                    call = worked_dict.get("Call")
                    if call in worked_list:
                        bandlist = worked_list[call]
                        bandlist.append(worked_dict["Band"])
                        worked_list[call] = bandlist
                        continue
                    worked_list[call] = [worked_dict["Band"]]
                return worked_list
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def get_ops(self) -> list:
        """get dict of unique station operators for contest"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select DISTINCT(Operator) from DXLOG where ContestNR = {self.current_contest};"
                )
                return cursor.fetchall()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def get_unique_band_and_mode(self) -> dict:
        """get count of unique band and mode as {mult: x}"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(DISTINCT(band || ':' || mode)) as mult from dxlog where ContestNR = {self.current_contest};"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def get_statistics(self) -> dict:
        """Get QSO count, sum of modes and sum of points in each band

        Band	QSO	CW	PH	DI	Pts
        1.8     9	9	0	0	18
        3.5     2	0	2	0	2
        7.0     33	30	3	0	60
        14.0	141	124	11	0	266
        21.0	128	84	32	0	255
        28.0	31	15	6	0	80
        144.0	4	4	0	0	2

        """

        query = """select 
                    Band,
                    count(*) as QSO,
                    sum(sortedmode.mode == 'CW') as CW, 
                    sum(sortedmode.mode == 'PH') as PH, 
                    sum(sortedmode.mode == 'DI') as DI,
                    sum(Points) as Points
                from (
                    select 
                        Band,
                        Points,
                        CASE 
                            WHEN Mode IN ('LSB','USB','SSB','FM','AM') THEN 'PH' 
                            WHEN Mode IN ('CW','CW-R') THEN 'CW' 
                            WHEN Mode In ('FT8','FT4','RTTY','PSK31','FSK441','MSK144','JT65','JT9','Q65', 'PKTUSB', 'PKTLSB') THEN 'DI' 
                            ELSE 'OTHER' 
                        END mode 
                    from DXLOG
                    ) as sortedmode
                    
                GROUP by Band
                ORDER by Band;
                """
        try:
            with sqlite3.connect(self.database) as conn:
                # conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(query)
                return cursor.fetchall()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def exec_sql(self, query: str) -> dict:
        """Exec one off queries returning one dict"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(query)
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def exec_sql_mult(self, query: str) -> list:
        """Exec one off queries returning list of dicts"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(query)
                return cursor.fetchall()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return ()

    def check_dupe_on_period_1_mode(
        self, call, band, mode, contest_start_time, contest_time_period_1
    ) -> dict:
        """Checks if a call is dupe on band/mode"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as isdupe from dxlog where Call = '{call}' and Mode = '{mode}' and Band = '{band}' and ContestNR = {self.current_contest} AND TS >= '{contest_start_time}' AND TS <= '{contest_time_period_1}';"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def check_dupe_on_period_2_mode(
        self,
        call,
        band,
        mode,
        contest_start_time,
        contest_time_period_1,
        contest_time_period_2,
    ) -> dict:
        """Checks if a call is dupe on band/mode"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as isdupe from dxlog where Call = '{call}' and Mode = '{mode}' and Band = '{band}' and ContestNR = {self.current_contest} AND TS >= '{contest_time_period_1}' AND TS <= '{contest_time_period_2}';"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}

    def check_dupe_on_period_3_mode(
        self,
        call,
        band,
        mode,
        contest_start_time,
        contest_time_period_2,
        contest_time_period_3,
    ) -> dict:
        """Checks if a call is dupe on band/mode"""
        try:
            with sqlite3.connect(self.database) as conn:
                conn.row_factory = self.row_factory
                cursor = conn.cursor()
                cursor.execute(
                    f"select count(*) as isdupe from dxlog where Call = '{call}' and Mode = '{mode}' and Band = '{band}' and ContestNR = {self.current_contest} AND TS >= '{contest_time_period_2}' AND TS <= '{contest_time_period_3}';"
                )
                return cursor.fetchone()
        except sqlite3.OperationalError as exception:
            print(f"{exception}")
            return {}
